#' Scorecards load metrics from verification suite
#' 
#'@description Scorecards function to load saved data files 
#'
#'@param system a vector of character strings defining the names of the 
#'  system names following the archive.yml format from verification suite 
#'@param reference a vector of character strings defining the names of 
#'  the references following the archive.yml format from verification suite 
#'  Pending to be test with more than one.
#'@param var a character string following the format from 
#'  variable-dictionary.yml from verification suite (TO DO: multiple variables)
#'@param start.year a numeric indicating the start year of the reference period
#'@param end.year a numeric indicating the end year of the reference period
#'@param m a vector of character strings indicating the start months
#'@param input.path a character string indicating the path where metrics output 
#'  files from verification suite where store (or any other compatible files)
#'
#'@return
#' loaded_metrics is a list by system and reference containing an array of with 
#'   the following dimensions: longitude, latitude, forecast months, metrics, 
#'   start dates.

## TO DO:
## other parameters? forecast time in case only one session or few months want to be displayed 

#'@example
#'recipe_file <- "recipes/recipe_1_nadia_scorecards.yml"
#'recipe <- read_yaml(recipe_file)
#'loaded_metrics <- SC_load_metrics(system = c('ecmwfs5','dwds21'),
#'                                 reference. = 'era5',
#'                                 var = 'tas',
#'                                 start.year = 1993,
#'                                 end.year = 2016,
#'                                 metrics =  c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr'),
#'                                 start.months = sprintf("%02d", 1:12),
#'                                 forecast.months = 1:6,
#'                                 input.path = '/esarchive/scratch/nmilders/scorecards_data/input_data')


SC_load_metrics <- function(system, 
                            reference, 
                            var, 
                            start.year, 
                            end.year,
                            metrics = NULL,
                            start.months = sprintf("%02d", 1:12), 
                            forecast.months,
                            input.path) {
  
  ## recipe: read recipe to obtain data of system, variable, start date etc.
  ## m: start_months to load from metrics outputs files
  ## metrics
  if (is.null(metrics)) {
    metrics <- c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr') ## TO DO: read metrics from input
  }
  ## system
  if (!is.list(system)) {
    if  (!is.character(system)) {
      stop("Parameter 'system' must be a character or a list.")
    }
  } 
  ## input.path: 
  if (is.null(input.path)) {
     stop("Parameter 'input.path' cannot be NULL.")
  }
  if (!is.character(input.path)) {
    stop("Parameter 'input.path must be a character string.")
  }
  if (length(input.path) > 1) {
    input.path <- input.path[1]
    warning("Parameter 'input.path' has length greater than 1 and only the ",
            "first element will be used.")
  }
  if (is.null(start.year)) {
    stop("Parameter 'start.year' cannot be NULL.")
  }
  if (is.null(end.year)) {
    stop("Parameter 'end.year' cannot be NULL.")
  }

  period <- paste0(start.year, "-", end.year)
  
  ## Define empty list to saved data
  all_metrics <- list()
  ## Load data for each system
  for (sys in 1:length(system)) {
    ## Define empty list to saved data
    by_reference <- list()
    ## Load data for each reference
    for (ref in 1:length(reference)) {
       ## Call function to load metrics data
       met <- initialfunction_read_data( 
                                   input.path = input.path, # recipe$Run$output, 
                                   system = system[sys],
                                   reference = reference[ref],
                                   var = var,
                                   period = period, 
                                   start.months = start.months,
                                   forecast.months = forecast.months,
                                   metrics = metrics)
       ## Save metric data as array in reference list
       by_reference <- append(by_reference, list(met)) 
       ## Include reference name
       names(by_reference)[[length(by_reference)]] <- reference[ref]
    } ## close loop on reference
    
    ## Save reference data in list of system
    all_metrics <- append(all_metrics, list(by_reference)) 
    ## Include system name
    names(all_metrics)[[length(all_metrics)]] <- system[sys]
  } ## close loop on system
  return(all_metrics)
} ## close function


##### Function to load data files #####

# metrics is a vector indicating the metrics to load using the names in the file 
# metrics is null by default and is currently hardcoded 
# Test:
#test <- initialfunction_read_data('/esarchive/scratch/nmilders/scorecards_data/input_data',
#                          "ecmwfs5", "era5", "tas", "1993-2016", start.months = sprintf("%02d", 1:12),
#                          metrics = c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr'))
# @return metrics in the order of the parameter metrics

initialfunction_read_data <- function(input.path, system, reference, 
                                      var, period, start.months, forecast.months, metrics){

    ## Define empty arrays
    lon <- NULL
    lat <- NULL
    met_by_grid <- NULL
    file_dim <- NULL
    
    ## Load data for each start date
    for(m in start.months){
      
     ## netCDF data file path:
     print(paste0("Loading netCDF file: ", paste0(input.path, "/", system, "/", var,
                                                "/scorecards_", system, "_", reference, "_",
                                                 var, "_", period, "_s", m, ".nc")))
     nc <- paste0(input.path, "/", system, "/", var, 
                           "/scorecards_", system, "_", reference, "_", 
                           var, "_", period, "_s", m, #mod.pressure,
                           ".nc")
     
      ## Load data of each metric:
      met_by_sdate <- NULL 
      for (kk in 1:length(metrics)) {
         met <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = metrics[kk], unlist = T, 
                                    drop_var_dim = T)},
                          error = function(cond) {
                            print("Missing file for start date")
                            message(cond)
                            return(NA)
                          })
         ## Save data for each metric
         attributes(met)$variables <- NULL
         if (is.null(file_dim)) {
           file_dim <- names(dim(met))
         }
         met_by_sdate <- abind::abind(met_by_sdate, met, along = length(dim(met)) + 1) 
      } ## close loop on metric
      
      ## Load latitude and longitude data as attribute of system
      if (is.null(lon)) {
        lon <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = 'longitude', unlist = T, drop_var_dim = T)})
        lat <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = 'latitude', unlist = T, drop_var_dim = T)})
      }
    ## Save metric data for each start date for each system  
    met_by_grid <- abind::abind(met_by_grid, met_by_sdate, along = length(dim(met_by_sdate)) + 1)
    } ## close loop on start months
    
  ## Add dimension names
  names(dim(met_by_grid)) <- c(file_dim, 'metric', 'sdate')
    
  ## Include data atttributes
  attributes(met_by_grid)$lon <- lon
  attributes(met_by_grid)$lat <- lat
  attributes(met_by_grid)$metrics <- metrics
  attributes(met_by_grid)$start.months <- start.months
  attributes(met_by_grid)$forecast.months <- forecast.months
  return(met_by_grid)
  
 } ## close function 

   
  