#' Scorecards spatial aggregation of loaded metrics
#' 
#'@description Scorecards function to perform the spatial aggregation of the 
#'  loaded metrics for the specified regions.
#'
#'@param loaded_metrics is a list of arrays containing the metrics loaded by the
#'  function SC_load_metrics.
#'@param metrics is a vector of the metrics to by aggregated, it should be the 
#'  same vector as introduced in the function SC_load_metrics.
#'@param region is a named list of vectors containing the desired regions to 
#'  analyze. For each region the following should be specified in this order:
#'  lon_min, lon_max, lat_min, lat_max.
#'@param ncores is the number of cores to use for the calculation.
#'
#'@example
#'aggregated_metrics <- SC_spatial_aggregation(loaded_metrics,
#'                                             metrics = c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr'), 
#'                                             regions = list(global = c(lon.min = 0, lon.max = 360, lat.min = -90, lat.max = 90),
#'                                                          europe = c(lon.min = -10, lon.max = 40, lat.min = 30, lat.max = 70)),
#'                                             ncores = 4)


SC_spatial_aggregation <- function(loaded_metrics,
                                   metrics,
                                   regions,
                                   ncores){

      file_dim <- NULL
    
    ## Loop over system
    all_metric_means <- NULL
    for (sys in 1:length(loaded_metrics)){
      ## Loop over reference
      means_by_ref <- NULL
      for (ref in 1:length(loaded_metrics[[sys]])){
        ## Loop over region
         means_by_reg <- NULL
         for (reg in 1:length(regions)){

                ## Get latitude and longitude from attributes of loaded metrics
                lat_dim_name <- attributes(attributes(attributes(loaded_metrics[[sys]][[ref]])$lat)$dim)$names
                lon_dim_name <- attributes(attributes(attributes(loaded_metrics[[sys]][[ref]])$lon)$dim)$names

                ## Calculate weighted means for defined regions for each system and reference
                weighted.mean <- multiApply::Apply(data = loaded_metrics[[sys]][[ref]], 
                                                   target_dims = c(lon_dim_name, lat_dim_name),
                                                   fun = ClimProjDiags::WeightedMean,
                                                   lon = as.vector(attributes(loaded_metrics[[sys]][[ref]])$lon), 
                                                   lat = as.vector(attributes(loaded_metrics[[sys]][[ref]])$lat),
                                                   mask = NULL, londim = lon_dim_name, latdim = lat_dim_name,
                                                   region = regions[[reg]],
                                                   ncores = ncores)$output1
            
                
                ## Save aggregated metric data
                if (is.null(file_dim)){
                  file_dim <- names(dim(weighted.mean))
                  }
                means_by_reg <- abind::abind(means_by_reg, weighted.mean, along = length(dim(weighted.mean)) + 1) 
            } ## close loop on region
        
          ## save data by reference
          means_by_ref <- abind::abind(means_by_ref, means_by_reg, along = length(dim(means_by_reg)) + 1)
        } ## close loop on reference
        
        ## Save data by system
        all_metric_means <- abind::abind(all_metric_means, means_by_ref, along = length(dim(means_by_ref)) + 1)
        
        ## Add dimension names
        names(dim(all_metric_means)) <- c(file_dim, 'region', 'reference', 'system')
        attributes(all_metric_means)$metrics <- metrics
        attributes(all_metric_means)$start.months <- attributes(loaded_metrics[[1]][[1]])$start.months
        attributes(all_metric_means)$forecast.months <- attributes(loaded_metrics[[1]][[1]])$forecast.months
        attributes(all_metric_means)$regions <- regions
      } ## close loop on system
    
    ## reorder dimensions in array
    all_metric_means <- Reorder(all_metric_means, c(6,5,2,1,3,4))
    attributes(all_metric_means)$system.name <- names(loaded_metrics)
    attributes(all_metric_means)$reference.name <- names(loaded_metrics[[1]])
    
    return(all_metric_means)
      
} ## close function
    
