#'Function to create all multi system/reference scorecards
#' 
#'@description Scorecards function to create scorecard tables for multiple systems
#' and references (types 9 to 12).
#'@param input_data is an array of spatially aggregated metrics containing the
#'   following dimensions; system, reference, metric, region, sdate, ftime.
#'@param system a vector of character strings defining the systems following the
#'   archive.yml format from verification suite 
#'@param reference a vector of character strings defining the references 
#'  following the archive.yml format from verification suite 
#'@param var a character string following the format from 
#'  variable-dictionary.yml from verification suite 
#'@param start.year a numeric indicating the start year of the reference period
#'@param end.year a numeric indicating the end year of the reference period
#'@param start.date a vector of character strings indicating the start months
#'@param forecast.month a vector of numeric indicating the forecast months
#'@param region.names a vector of character strings containing names of the 
#'  regions corresponding to the input data
#'@param metrics a vector of character strings containing the metrics,  
#'  NOTE: must be in same order as in data.
#'@param fileout a path of the location to save the scorecard plots

#'@example
#' scorecard_table <- SC_create_table(input_data = aggregated_metrics,
#'                                    system.name = c('ecmwfs5','dwds21'),
#'                                    reference.name = 'era5',
#'                                    var = 'tas', 
#'                                    start.year = 1993,
#'                                    end.year = 2016,
#'                                    start.months = 1:12,
#'                                    forecast.months = 1:6,
#'                                    region.names = c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
#'                                    metrics = c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr'),
#'                                    fileout = '/esarchive/scratch/nmilders/scorecards_data/saved_results'
#'                                    )


SC_create_table_multi <- function(input_data,
                                  system,
                                  reference,
                                  var,
                                  start.year,
                                  end.year,
                                  start.months,
                                  forecast.months,
                                  region.names,
                                  metrics,
                                  fileout){
  
  ## For testing
  # input_data <- aggregated_metrics
  # system <- c('ecmwfs5', 'dwds21')
  # reference <-'era5'
  # var <- 'tas'
  # start.year <- 1993
  # end.year <- 2016
  # start.months <- sprintf("%02d", 1:12)
  # forecast.months <- 1:6
  # region.names <- c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
  # metrics <- c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr')
  # fileout <- '/esarchive/scratch/nmilders/scorecards_data/saved_results'
  
  setwd("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/")
  
  ## Load functions
  source("./modules/Scorecards/SC_3_transform_data.R")
  source("./modules/Scorecards/SC_plot_multi_by_region.R")
  source("./modules/Scorecards/SC_plot_multi_by_forecast_month.R")
  
  ## Checks to apply:
  # first dimension in aggregated_metrics is system and second dimension is reference
  # either multi-system and one reference, or multi-reference and one system
  
  # start.months <- attributes(data)$start.months
  # regions.names <- names(attributes(data)$regions)
  # forecast.months <- attributes(data)$forecast.months  
  # reference <- attributes(data)$reference
  # system <- attributes(data)$system[1] ## selecting only first system for testing
  
  ## Make sure input_data is in correct order for using in functions:
  data_order <- c('system','reference','metric','time','sdate','region')
  input_data <- Reorder(input_data, data_order)
  
  # Transform data for scorecards by forecast month (types 2 & 4)
  transformed_data <- SC_transform_data(data = input_data,
                                        sdate_dim = 'sdate',
                                        ftime_dim = 'time')
  
  ## TO DO: 
  ## Need to select single reference for multiple systems
  ## or single system for multiple references

    #### Scorecard_type 9 ####
    ## (no transformation or reorder)
    data_sc_9 <- input_data
    SC_plot_multi_by_region(data = data_sc_9,
                             system = system,
                             reference = reference,
                             var = var,
                             start.year = start.year,
                             end.year = end.year,
                             start.months = start.months,
                             forecast.months = forecast.months,
                             region.names = region.names,
                             metrics = metrics,
                             scorecard.type = 9,
                             fileout =  fileout)
    
    
    #### Scorecard_type 10 ####
    ## (reorder only)
    new_order <- c(1,2,3,6,5,4) 
    data_sc_10 <- Reorder(input_data, new_order)
    SC_plot_multi_by_forecast_month(data = data_sc_10,
                                     system = system,
                                     reference = reference,
                                     var = var,
                                     start.year = start.year,
                                     end.year = end.year,
                                     start.months = start.months,
                                     forecast.months = forecast.months,
                                     region.names = region.names,
                                     metrics = metrics,
                                     scorecard.type = 10,
                                     fileout =  fileout)

    #### Scorecard_type 11 ####
    ## (transformation only)
    data_sc_3 <- transformed_data
    SC_plot_multi_by_region(data = data_sc_3,
                             system = system,
                             reference = reference,
                             var = var,
                             start.year = start.year,
                             end.year = end.year,
                             start.months = start.months,
                             forecast.months = forecast.months,
                             region.names = region.names,
                             metrics = metrics,
                             scorecard.type = 11,
                             fileout =  fileout)
    
    
    #### Scorecard_type 12 ####
    ## (transformation and reorder)
    new_order <- c(1,2,3,6,5,4)
    data_sc_12 <- Reorder(transformed_data, new_order)
    SC_plot_multi_by_forecast_month(data = data_sc_12,
                                     system = system,
                                     reference = reference,
                                     var = var,
                                     start.year = start.year,
                                     end.year = end.year,
                                     start.months = start.months,
                                     forecast.months = forecast.months,
                                     region.names = region.names,
                                     metrics = metrics,
                                     scorecard.type = 12,
                                     fileout =  fileout)

  print("All multi scorecard plots created")
  
} ## close function      

