#' Scorecards function create simple scorecards by forecast month (types 2 & 4)
#' 
#'@description function to create a scorecard for a single system and reference 
#'  combination, showing data by forecast month and region.
#'  
#'@param data is an array of spatially aggregated metrics data, containing the
#'   following dimensions; metric, region, sdate, ftime.
#'@param system a character string defining the system following the
#'   archive.yml format from verification suite 
#'@param reference a character string defining the name of the reference 
#'  following the archive.yml format from verification suite 
#'@param var a character string following the format from 
#'  variable-dictionary.yml from verification suite 
#'@param start.year a numeric indicating the start year of the reference period
#'@param end.year a numeric indicating the end year of the reference period
#'@param start.date a vector of character strings indicating the start months
#'@param forecast.month a vector of numeric indicating the forecast months
#'@param region.names a vector of character strings containing names of the 
#'  regions corresponding to the input data
#'@param metrics a vector of character strings containing the metrics,  
#'  NOTE: must be in same order as in data.
#'@param scorecard.type a numeric indicating the type of scorecard to plot
#'@param fileout a path of the location to save the scorecard plots
#'  
#'@example 
#'scorecard_plot <- SC_plot_simple_by_forecast_month(data = aggregated_metrics[1,1,,,,],
#'                                                   system = 'ecmwfs5' ,
#'                                                   reference = 'era5',
#'                                                   var = 'tas',
#'                                                   start.year = 1993,
#'                                                   end.year = 2016,
#'                                                   start.months = 1:12,
#'                                                   forecast.months = 1:6,
#'                                                   region.names = c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
#'                                                   metrics = c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr'),
#'                                                   scorecard.type = 2,
#'                                                   fileout = '/esarchive/scratch/nmilders/scorecards_data/saved_results'
#'                                                   )


SC_plot_simple_by_forecast_month <- function(data,
                                             system,
                                             reference,
                                             var,
                                             start.year,
                                             end.year,
                                             start.months,
                                             forecast.months,
                                             region.names,
                                             metrics,
                                             scorecard.type,
                                             fileout){
  

  # ## For testing:
  # data <- Reorder(aggregated_metrics[1,1,,,,], c(1,4,3,2))
  # system <- c('ecmwfs5', 'dwds21')
  # reference <-'era5'
  # var <- 'tas'
  # start.year <- 1993
  # end.year <- 2016
  # start.months <- 1:12
  # forecast.months <- 1:6
  # region.names <- c('Tropics', 'Extra-tropical NH', 'Extra-tropical SH')
  # metrics <- c('mean_bias', 'enscorr', 'rpss','crpss', 'enssprerr')
  # scorecard.type <- 2
  # fileout <- '/esarchive/scratch/nmilders/scorecards_data/saved_results/testing'
  
  ## Load configuration files
  archive <- read_yaml("./conf/archive.yml")$archive
  var_dict <- read_yaml("./conf/variable-dictionary.yml")$vars
  
  ## Load functions
  source("./modules/Scorecards/SC_breaks.R")
  source("./modules/Scorecards/SC_table_colors.R")
  source("./modules/Scorecards/SC_legend.R")

  ## Round data
  data <- round(data,2)
  
  ## get dimensions of inputs
  n.start.months <- as.numeric(dim(data)["sdate"]) ## TO DO: create sdate_dim
  n.ftime.months <- as.numeric(dim(data)["time"]) ## TO DO: create ftime_dim
  n.metrics <- length(metrics) 
  n.regions <- length(region.names) 
  
  ## Define table size
  n.rows <- n.regions * n.ftime.months 
  n.columns <- 2 + (n.metrics * n.start.months)

  ## Get display names from configuration files
  var.name <- var_dict[[var]]$long_name
  var.units <- var_dict[[var]]$units
  system.name <- archive$System[[which(archive$System %like% system)]]$system_name
  reference.name <- archive$Reference[[reference]]$reference_name
  
  metric.names.list <- list(mean_bias = paste0("Mean bias (", var.units,")"),
                            enscorr = "Correlation",
                            rpss = "RPSS",
                            crpss = "CRPSS",
                            enssprerr = "Spread-to-error ratio")
  for(met in metrics){
    if(met == metrics[1]){
      metric.names <- metric.names.list[[met]] 
    } else {
      metric.names <- c(metric.names, metric.names.list[[met]])
    }
  }
  
  ## Region layout for plotting 
  forecast.month.figure <- rep("", n.regions * n.ftime.months)
  for(ft in 1:n.ftime.months) forecast.month.figure[(ft-1) * n.regions + 1] = ft

  ## Define table columns
   column.names <- c("Forecast month", "Region", rep(c(month.abb[as.numeric(start.months)]), n.metrics))

  if(scorecard.type == 2){
    column.header <- "Start date"
  }
  if(scorecard.type == 4){
    column.header <- "Target month"
  }
   
   options(stringsAsFactors = FALSE) 
   table.title <- data.frame(c1=paste0(var.name, " of ", system.name), c2 = n.columns)
   header.names <- as.data.frame(data.frame(c1=c("", metric.names), c2=c(2, rep(n.start.months, n.metrics)))) ## TO DO: need to include metric units
   header.names2 <- as.data.frame(data.frame(c1=c("", paste0(rep(column.header, n.metrics))), c2=c(2, rep(n.start.months, n.metrics))))
   ref.label <- data.frame(c1=paste0("(Ref: ", reference.name, " ", start.year, "-", end.year, ")"), c2 = n.columns)
   
   ## Define data inside the scorecards table
   ## NEED TO FIX: allow different or of metrics than in data
   for(ft in 1:n.ftime.months){
     table_metric <- data.frame(table_column_2 = as.character(region.names))
     for(met in 1:n.metrics){
       table_metric <- data.frame(table_metric, data[met,,,ft])   
     }
     if(ft == 1){
       table_ft <- table_metric
     } else {
       table_ft <- rbind(table_ft, table_metric)
     }
   } ## close loop on ft

   ## All data for plotting in table
   table <- data.frame(table_column_1 = forecast.month.figure, table_ft)
   
   ## Find position of mean bias metric to calculate breaks
   if ('mean_bias' %in% metrics) {
     pos_bias <- which(metrics == 'mean_bias')
     breaks_bias <- SC_breaks(Subset(data, along = 'metric', 
                                     indices = pos_bias))
   }
   
   ## Define breaks for each metric based of metric position:
   metric.breaks <- list('mean_bias' = breaks_bias,
                         'enscorr' = c(-1, -0.8, -0.6, -0.4, -0.2, 0, 0.2, 0.4, 0.6, 0.8, 1),
                         'rpss' = c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5),
                         'crpss' = c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5),
                         'enssprerr' = c(0, 0.6, 0.7, 0.8, 0.9, 1, 1.2, 1.4, 1.6, 1.8, 2))
   
   ## Set table background colors
   palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
   colorunder <- "#04040E"
   colorsup <- "#730C04"
   
   metric.col <- array(colorunder, c(n.ftime.months * n.regions, n.columns-2)) 
   metric.text.col <- array("#2A2A2A", c(n.ftime.months * n.regions, n.columns-2)) 
   # metric.text.bold <- array(TRUE, c(n.ftime.months * n.regions, n.columns-2)) ## Setting all values to bold
 
   ## Define colors to show in table
   table_colors  <- SC_table_colors(table,
                                    n.rows,
                                    n.columns,
                                    metrics,
                                    metric.breaks,
                                    n.start.months,
                                    palette,
                                    colorunder,
                                    colorup,
                                    metric.col,
                                    metric.text.col)
   
   metric.col <- table_colors$metric.col
   metric.text.col <- table_colors$metric.text.col
   # metric.text.bold <- table_colors$metric.text.bold
   
   ## Create HTML table
   table.html.part <- list()
   table.html.part[[1]] <- kbl(table, escape = F, col.names = column.names, align = rep("c", n.columns)) %>%
     kable_paper("hover", full_width = F) %>%
     add_header_above(header = header.names2, font = 14)%>%
     add_header_above(header = ref.label, align = "left", font_size = 14) %>% 
     add_header_above(header = header.names, font_size = 20) %>%
     add_header_above(header = table.title, font_size = 20, align = "left")
   
   for(i in 1:n.metrics){ 
     for(stm in 1:n.start.months){ 
       my.background <- metric.col[(i - 1) * n.start.months + stm]
       my.text.color <- metric.text.col[(i - 1) * n.start.months + stm ]
       #my.bold <- metric.text.bold[(i - 1) * n.start.months + stm]
       
       table.html.part[[(i - 1) * n.start.months + stm + 1]] <-
         column_spec(table.html.part[[(i - 1) * n.start.months + stm]], 2 + n.start.months * (i - 1) + stm, 
                     background = my.background[1:n.rows,], 
                     color = my.text.color[1:n.rows,], 
                     bold = T) ## strsplit(toString(bold), ', ')[[1]]
     } ## close loop on stm
   } ## close loop on i
   
   n.last.list <- n.metrics * n.start.months + 1
   
   column.borders <- NULL
   
   for(i in 1:n.metrics){
      column.spacing <- (n.start.months * i) + 2
      column.borders <- c(column.borders, column.spacing)
   }
   
   table.html <- column_spec(table.html.part[[n.last.list]], 1, bold = TRUE, width_min = "1cm", width_max = "1cm") %>%
     column_spec(1, border_left = "4px solid black") %>% 
     column_spec(2 , bold = TRUE, width_min = "4cm", width_max = "4cm") %>%
     column_spec(3:n.columns, width_max = "1.1cm", width_min = "1.1cm") %>% 
     column_spec(c(1, 2, column.borders), border_right = "2px solid black") %>% 
     column_spec(n.columns, border_right = "4px solid black") %>% 
     row_spec(seq(from = 0, to = n.ftime.months * n.regions, by = n.regions), extra_css = "border-bottom: 2px solid black", hline_after = TRUE)
   
   currentworkingdir <- getwd()
   
   ## Create color bar legend
   scorecard_legend <- SC_legend(table.html,
                                 metrics,
                                 metric.breaks,
                                 palette,
                                 colorunder,
                                 colorup,
                                 fileout)
   
   ## Save scorecard png in desired directory
   dirout <- save_scorecard_single(system, reference, var, start.year, end.year, scorecard.type)
   system(paste0("mv ", './scorecard.png', paste0(" ", dirout))) 
   print(paste0("Scorecard type ", scorecard.type, " figure saved (sys: ", system.name, " & ref: ", reference.name, ")" ))
   
   setwd(currentworkingdir)
 
}## close function
