#G# TODO: Remove once released in s2dv/CSTools
source("modules/Visualization/tmp/PlotMostLikelyQuantileMap.R")
source("modules/Visualization/tmp/PlotCombinedMap.R")

## TODO: Add the possibility to read the data directly from netCDF
## TODO: Adapt to multi-model case
## TODO: Add param 'raw'?

plot_data <- function(recipe,
		      data,
		      calibrated_data = NULL,
		      skill_metrics = NULL,
		      probabilities = NULL,
		      archive = NULL,
		      significance = F) {

  # Try to produce and save several basic plots.
  # recipe: the auto-s2s recipe as read by read_yaml()
  # archive: the auto-s2s archive as read by read_yaml()
  # data: list containing the hcst, obs and (optional) fcst s2dv_cube objects
  # calibrated_data: list containing the calibrated hcst and (optional) fcst
  # s2dv_cube objects
  # skill_metrics: list of arrays containing the computed skill metrics
  # significance: Bool. Whether to include significance dots where applicable

  outdir <- paste0(get_dir(recipe), "/plots/")
  dir.create(outdir, showWarnings = FALSE, recursive = TRUE)

  if ((is.null(skill_metrics)) && (is.null(calibrated_data)) && 
      (is.null(data$fcst))) {
    stop("The Visualization module has been called, but there is no data ",
	 "that can be plotted.")
  }

  if (is.null(archive)) {
    if (tolower(recipe$Analysis$Horizon) == "seasonal") {
      archive <- read_yaml(paste0(recipe$Run$code_dir,
				  "conf/archive.yml"))$archive
    } else if (tolower(recipe$Analysis$Horizon) == "decadal") {
      archive <- read_yaml(paste0(recipe$Run$code_dir,
				   "conf/archive_decadal.yml"))$archive
    }
  }

  # Plot skill metrics
  if (!is.null(skill_metrics)) {
    plot_skill_metrics(recipe, archive, data$hcst, skill_metrics, outdir,
		       significance)
  }

  # Plot forecast ensemble mean
  if (!is.null(calibrated_data$fcst)) {
    plot_ensemble_mean(recipe, archive, calibrated_data$fcst, outdir)
  } else if (!is.null(data$fcst)) {
    warn(recipe$Run$logger, "Only the uncalibrated forecast was provided. 
	 Using this data to plot the forecast ensemble mean.")
    plot_ensemble_mean(recipe, archive, data$fcst, outdir)
  }

  # Plot Most Likely Terciles
  if ((!is.null(probabilities)) && (!is.null(calibrated_data$fcst))) {
    plot_most_likely_terciles(recipe, archive, calibrated_data$fcst,
			      probabilities$percentiles, outdir)
  } else if ((!is.null(probabilities)) && (!is.null(data$fcst))) {
    warn(recipe$Run$logger, "Only the uncalibrated forecast was provided.
	 Using this data to plot the most likely terciles.")
    plot_most_likely_terciles(recipe, archive, data$fcst,
			      probabilities$percentiles, outdir)
  }
}

plot_skill_metrics <- function(recipe, archive, data_cube, skill_metrics,
			       outdir, significance = F) {

  # Abort if frequency is daily
  if (recipe$Analysis$Variables$freq == "daily_mean") {
    stop("Visualization functions not yet implemented for daily data.")
  }
  # Abort if skill_metrics is not list
  if (!is.list(skill_metrics) || is.null(names(skill_metrics))) {
    stop("The element 'skill_metrics' must be a list of named arrays.")
  }
  
  latitude <- data_cube$lat
  longitude <- data_cube$lon
  system_name <- archive$System[[recipe$Analysis$Datasets$System$name]]$name
  hcst_period <- paste0(recipe$Analysis$Time$hcst_start, "-",
			recipe$Analysis$Time$hcst_end)
  init_month <- lubridate::month(as.numeric(substr(recipe$Analysis$Time$sdate,
					           start = 1, stop = 2)),
				 label = T, abb = T)
  
  # Group different metrics by type
  skill_scores <- c("rpss", "bss90", "bss10", "frpss", "crpss", "mean_bias_ss",
		    "enscorr", "rpss_specs", "bss90_specs", "bss10_specs",
		    "enscorr_specs")
  scores <- c("rps", "frps", "crps", "frps_specs")

  for (name in c(skill_scores, scores, "mean_bias", "enssprerr")) {

    if (name %in% names(skill_metrics)) {
      # Define plot characteristics and metric name to display in plot
      if (name %in% c("rpss", "bss90", "bss10", "frpss", "crpss",
		      "rpss_specs", "bss90_specs", "bss10_specs")) {
	display_name <- toupper(strsplit(name, "_")[[1]][1])
        skill <- skill_metrics[[name]]
	brks <- seq(-1, 1, by = 0.1)
	col2 <- grDevices::hcl.colors(length(brks) - 1, "RdBu", rev = TRUE)

      } else if (name == "mean_bias_ss") {
	display_name <- "Mean Bias Skill Score"
	skill <- skill_metrics[[name]]
	brks <- seq(-1, 1, by = 0.1)
	col2 <- grDevices::hcl.colors(length(brks) - 1, "RdBu", rev = TRUE)

      } else if (name %in% c("enscorr", "enscorr_specs")) {
	display_name <- "Ensemble Mean Correlation"
	skill <- skill_metrics[[name]]
	brks <- seq(-1, 1, by = 0.1)
	col2 <- grDevices::hcl.colors(length(brks) - 1, "RdBu", rev = TRUE)

      } else if (name %in% scores) {
	skill <- skill_metrics[[name]]
	display_name <- toupper(strsplit(name, "_")[[1]][1])
        brks <- seq(0, 1, by = 0.1)
        col2 <- grDevices::hcl.colors(length(brks) - 1, "Reds")

      } else if (name == "enssprerr") {
        ## TODO: Adjust colorbar parameters
	skill <- skill_metrics[[name]]
	display_name <- "Spread-to-Error Ratio"
        brks <- pretty(0:max(skill, na.rm = T), n = 20, min.n = 10)
	col2 <- grDevices::hcl.colors(length(brks) - 1, "RdBu", rev = TRUE)

      } else if (name == "mean_bias") {
	skill <- skill_metrics[[name]]
	display_name <- "Mean Bias"
	max_value <- max(abs(skill))
	ugly_intervals <- seq(-max_value, max_value, (max_value*2)/10)
	brks <- pretty(ugly_intervals, n = 20, min.n = 10)
	col2 <- grDevices::hcl.colors(length(brks) - 1, "RdBu", rev = TRUE)
      }

    options(bitmapType = "cairo")

    # Reorder dimensions
    skill <- Reorder(skill, c("time", "longitude", "latitude"))
    # If the significance has been requested and the variable has it,
    # retrieve it and reorder its dimensions.
    significance_name <- paste0(name, "_significance")
    if ((significance) && (significance_name %in% names(skill_metrics))) {
      significance_name <- paste0(name, "_significance")
      skill_significance <- skill_metrics[[significance_name]]
      skill_significance <- Reorder(skill_significance, c("time", 
      						          "longitude",
      						          "latitude"))
      # Split skill significance into list of lists, along the time dimension
      # This allows for plotting the significance dots correctly.
      skill_significance <- ClimProjDiags::ArrayToList(skill_significance,
						       dim = 'time',
						       level = "sublist",
						       names = "dots")
    } else {
      skill_significance <- NULL
    }
    # Define output file name and titles
    outfile <- paste0(outdir, name, ".png")
    toptitle <- paste(display_name, "-", data_cube$Variable$varName,
      		 "-", system_name, "-", init_month, hcst_period)
    months <- unique(lubridate::month(data_cube$Dates$start,
      			              label = T, abb = F))
    titles <- as.vector(months)
    # Plot
    suppressWarnings(
    PlotLayout(PlotEquiMap, c('longitude', 'latitude'),
               asplit(skill, MARGIN=1), # Splitting array into a list
	       longitude, latitude,
	       special_args = skill_significance, 
      	       dot_symbol = 20,
      	       toptitle = toptitle,
      	       title_scale = 0.6,
      	       titles = titles,
      	       filled.continents=F,
               brks = brks,
               cols = col2,
               fileout = outfile,
               bar_label_digits = 3)
    )
    }    
  }

  info(recipe$Run$logger,
       "##### SKILL METRIC PLOTS SAVED TO OUTPUT DIRECTORY #####")
}

plot_ensemble_mean <- function(recipe, archive, fcst, outdir) {

  # Abort if frequency is daily
  if (recipe$Analysis$Variables$freq == "daily_mean") {
    stop("Visualization functions not yet implemented for daily data.")
  }

  latitude <- fcst$lat
  longitude <- fcst$lon
  system_name <- archive$System[[recipe$Analysis$Datasets$System$name]]$name
  variable <- recipe$Analysis$Variables$name
  units <- attr(fcst$Variable, "variable")$units
  start_date <- paste0(recipe$Analysis$Time$fcst_year,
		       recipe$Analysis$Time$sdate)

  # Compute ensemble mean
  ensemble_mean <- s2dv::MeanDims(fcst$data, 'ensemble')
  # Drop extra dims, add time dim if missing:
  ensemble_mean <- drop(ensemble_mean)

  if (!("time" %in% names(dim(ensemble_mean)))) {
    dim(ensemble_mean) <- c("time" = 1, dim(ensemble_mean))
  }
  if (!'syear' %in% names(dim(ensemble_mean))) {
    ensemble_mean <- Reorder(ensemble_mean, c("time", "longitude", "latitude"))
  } else {
    ensemble_mean <- Reorder(ensemble_mean, c("syear", "time", "longitude", "latitude"))
  }
  ## TODO: Redefine column colors, possibly depending on variable
  if (variable == 'prlr') {
    palette = "BrBG"
    rev = F
  } else {
    palette = "RdBu"
    rev = T
  }

  brks <- pretty(range(ensemble_mean, na.rm = T), n = 15, min.n = 8)
  col2 <- grDevices::hcl.colors(length(brks) - 1, palette, rev = rev)
  # color <- colorRampPalette(col2)(length(brks) - 1)
  options(bitmapType = "cairo")

  for (i_syear in start_date) {
    # Define name of output file and titles
    if (length(start_date) == 1) {
      i_ensemble_mean <- ensemble_mean
      outfile <- paste0(outdir, "forecast_ensemble_mean.png")
    } else {
      i_ensemble_mean <- ensemble_mean[which(start_date == i_syear), , , ]
      outfile <- paste0(outdir, "forecast_ensemble_mean_", i_syear, ".png")
    }
    toptitle <- paste("Forecast Ensemble Mean -", variable, "-", system_name,
  	  	      "- Initialization:", i_syear)
    months <- lubridate::month(fcst$Dates$start[1, 1, which(start_date == i_syear), ],
                               label = T, abb = F)
    titles <- as.vector(months)
    # Plots
    PlotLayout(PlotEquiMap, c('longitude', 'latitude'),
   	       i_ensemble_mean, longitude, latitude,
               filled.continents = F,
	       toptitle = toptitle,
	       title_scale = 0.6,
	       titles = titles,
	       units = units,
	       cols = col2,
	       brks = brks,
	       fileout = outfile,
               bar_label_digits = 4)
  }

  info(recipe$Run$logger,
       "##### FCST ENSEMBLE MEAN PLOT SAVED TO OUTPUT DIRECTORY #####")
}

plot_most_likely_terciles <- function(recipe, archive,
				      fcst,
				      percentiles, 
				      outdir) {

  # Abort if frequency is daily
  if (recipe$Analysis$Variables$freq == "daily_mean") {
    stop("Visualization functions not yet implemented for daily data.")
  }

  latitude <- fcst$lat
  longitude <- fcst$lon
  system_name <- archive$System[[recipe$Analysis$Datasets$System$name]]$name
  variable <- recipe$Analysis$Variables$name
  start_date <- paste0(recipe$Analysis$Time$fcst_year,
		       recipe$Analysis$Time$sdate)
  if (is.null(recipe$Analysis$remove_NAs)) {
    recipe$Analysis$remove_NAs <- FALSE
  }
  if (is.null(recipe$Analysis$ncores)) {
    recipe$Analysis$ncores <- 1
  }

  # Compute probability bins for the forecast
  if (is.null(percentiles$percentile_33) | is.null(percentiles$percentile_33)) {
    stop("The quantile array does not contain the 33rd and 66th percentiles,",
	 " the most likely tercile map cannot be plotted.")
  }

  terciles <- abind(percentiles$percentile_33, percentiles$percentile_66,
		     along = 0)
  names(dim(terciles)) <- c("bin", names(dim(percentiles$percentile_33)))
  probs_fcst <- compute_probs(fcst$data, terciles,
			      ncores = recipe$Analysis$ncores,
			      na.rm = recipe$Analysis$remove_NAs)

  # Drop extra dims, add time dim if missing:
  probs_fcst <- drop(probs_fcst)
  if (!("time" %in% names(dim(probs_fcst)))) {
    dim(probs_fcst) <- c("time" = 1, dim(probs_fcst))
  }
  if (!'syear' %in% names(dim(probs_fcst))) {
    probs_fcst <- Reorder(probs_fcst, c("time", "bin", "longitude", "latitude"))
  } else {
    probs_fcst <- Reorder(probs_fcst, c("syear", "time", "bin", "longitude", "latitude"))
  }

  for (i_syear in start_date) {
    # Define name of output file and titles
    if (length(start_date) == 1) {
      i_probs_fcst <- probs_fcst
      outfile <- paste0(outdir, "forecast_most_likely_tercile.png")
    } else {
      i_probs_fcst <- probs_fcst[which(start_date == i_syear), , , , ]
      outfile <- paste0(outdir, "forecast_most_likely_tercile_", i_syear, ".png")
    }
    toptitle <- paste("Most Likely Tercile -", variable, "-", system_name, "-",
 		      "Initialization:", i_syear)
    months <- lubridate::month(fcst$Dates$start[1, 1, which(start_date == i_syear), ],
	  	               label = T, abb = F)
    ## TODO: Ensure this works for daily and sub-daily cases
    titles <- as.vector(months)

    # Plots
    ## NOTE: PlotLayout() and PlotMostLikelyQuantileMap() are still being worked
    ## on. 
    suppressWarnings(
    PlotLayout(PlotMostLikelyQuantileMap, c('bin', 'longitude', 'latitude'),
  	       cat_dim = 'bin',
  	       i_probs_fcst, longitude, latitude,
	       coast_width = 1.5,
	       title_scale = 0.6,
	       legend_scale = 0.8, #cex_bar_titles = 0.6,
	       toptitle = toptitle,
	       titles = titles,
	       fileout = outfile,
	       bar_label_digits = 2,
	       triangle_ends = c(F, F), width = 11, height = 8)
    )
  }

  info(recipe$Run$logger,
       "##### MOST LIKELY TERCILE PLOT SAVED TO OUTPUT DIRECTORY #####")
}
